from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from django.core.files.storage import default_storage
from lxml import objectify, etree

from . import LANGUAGES, INVALID_XML_ERROR


def construct_MultiLingualText_from_xml(xml, instance):
    u"""
    Arguments:
    `xml` : A block of XML formatted like this:
        <languages>
            <language code="en">
                Hello
            </language>
            <language code="es">
                Hola
            </language>
        </languages>

    * `instance`: A MultiLingualText instance

    If the above block of XML was passed to this function (as `xml`)
    `instance` will now have two attributes:

    * `en` with a value of 'Hello'
    * `es` with a value of 'Hola'
    """
    try:
        xml_as_python_object = objectify.fromstring(xml)
    except etree.XMLSyntaxError:
        raise Exception(INVALID_XML_ERROR + ' MultiLingualText')
    else:
        # Creating a dictionary of all the languages passed in the value XML
        # with the language code (i.e. 'en', 'de', 'fr') as the key
        text_dict = {}
        try:
            text_dict = dict(
                (unicode(l.get(u'code')), unicode(l.text or u''))
                for l in xml_as_python_object.language
            )
        except AttributeError:
            # Empty fields throw-off lxml and cause an AttributeError
            pass
        for code, verbose in LANGUAGES:
            setattr(instance, code, text_dict.get(code, u''))


def construct_MultiLingualFile_from_xml(xml, instance, storage=default_storage):
    u"""
    Arguments:
    `xml` : A block of XML formatted like this:
        <languages>
            <language code="en">
                path/to/file.ext
            </language>
            <language code="es">
                path/to/file2.ext
            </language>
        </languages>

    * `instance`: A MultiLingualFile instance

    If the above block of XML was passed to this function (as `xml`)
    `instance` will now have two attributes:
    * `en` with a file stored at path/to/file.ext within `storage`
    * `es` with a file stored at path/to/file2.ext within `storage`
    """
    from .datastructures import MultiLingualFieldFile
    try:
        xml_as_python_object = objectify.fromstring(xml)
    except etree.XMLSyntaxError:
        raise Exception(INVALID_XML_ERROR + ' MultiLingualText')
    else:
        # Creating a dictionary of all the languages passed in the value XML
        # with the language code (i.e. 'en', 'de', 'fr') as the key
        text_dict = {}
        try:
            text_dict = dict(
                (unicode(l.get('code')), unicode(l.text or u''))
                for l in xml_as_python_object.language
            )
        except AttributeError:
            # Empty fields throw-off lxml and cause an AttributeError
            pass
        for code, verbose in LANGUAGES:
            setattr(
                instance,
                code,
                MultiLingualFieldFile(
                    storage=storage,
                    name=text_dict[code]
                ) if code in text_dict else None
            )



from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from django.core.exceptions import ImproperlyConfigured
from django.core.files.base import File
from django.utils.encoding import smart_str
from django.utils.translation import get_language

from lxml import objectify, etree

from . import utils, LANGUAGES, INVALID_XML_ERROR, UNKNOWN_LANGUAGE_CODE_ERROR


class MultiLingualText(object):
    u"""
    A class that aggregates manually-written translations for
    the same piece of text.
    """

    def __init__(self, xml=None):
        u"""
        `xml` : An optional block of XML formatted like this:
        <languages>
            <language code="en">
                Hello
            </language>
            <language code="es">
                Hola
            </language>
        </languages>

        If the above block of XML was passed (as `xml`) to an instance of
        MultiLingualText that instance would have two attributes:

        * `en` with a value of 'Hello'
        * `es` with a value of 'Hola'

        If `xml` is not passed to a MultiLingualText instance an attribute for
        each language in settings.LANGUAGES will be built.
        """
        self.languages = LANGUAGES
        # Converting XML (passed-in as `xml`) to a python object via lxml
        if xml:
            try:
                utils.construct_MultiLingualText_from_xml(xml, self)
            except Exception:
                if not xml.startswith('<'):
                    for i, lang_tup in enumerate(LANGUAGES):
                        code, verbose = lang_tup
                        if i == 0:
                            val = xml
                        else:
                            val = u''
                        setattr(self, code, val)
                else:
                    raise
        else:
            for code, verbose in LANGUAGES:
                setattr(self, code, u'')

    def get_for_current_language(self):
        """
        Returns the attribute on this object associated with the current
        language of the active thread (as provided by
        django.utils.translation.get_language)
        """
        current = get_language()

        try:
            val = getattr(self, current)
        except AttributeError:
            if current not in [code for code, verbose in LANGUAGES]:
                raise_exception = True
                if '-' in current:
                    try:
                        val = getattr(self, current[:2])
                    except AttributeError:
                        pass
                    else:
                        raise_exception = False
                if raise_exception:
                    raise ImproperlyConfigured(
                        UNKNOWN_LANGUAGE_CODE_ERROR.format(current)
                    )
            else:
                val = ''
        return val

    def __repr__(self):
        val = self.get_for_current_language()
        return smart_str(val, errors='ignore')

    def __unicode__(self):
        val = self.get_for_current_language()
        return smart_str(val, errors='strict')

    def as_xml(self):
        u"""Returns this instance as XML."""
        xml_to_return = etree.Element(u'languages')
        for key, value in self.__dict__.iteritems():
            if key != u'languages':
                language = etree.Element(u'language', code=key)
                language.text = value
                xml_to_return.append(language)
        return etree.tostring(xml_to_return)

    def __nonzero__(self):
        u"""
        Provides 'truth value testing' to MultiLingualText instances
        based on the current language thread.

        Basically, if the language-specific attribute of a MultiLingualText
        instance associated with the current language thread would evaluate
        to `False` then calling `if` on the instance itself will evaluate to
        `False` as well.
        """
        if self.get_for_current_language():
            return True
        else:
            return False


class MultiLingualFieldFile(File):
    u"""
    A `File` subclasses used specifically for the language-keyed attributes
    of a MultiLingualFileField instance.

    Functions almost identically to django's FieldFile
    """
    def __init__(self, storage, name):
        super(MultiLingualFieldFile, self).__init__(None, name)
        self.name = name
        self.storage = storage
        self._committed = True

    @property
    def file(self):
        if not getattr(self, u'_file', None):
            self._file = self.storage.open(self.name, u'rb')
        return self._file

    @file.setter
    def file(self, value):
        self._file = value

    @file.deleter
    def file(self):
        del self._file

    @property
    def path(self):
        return self.storage.path(self.name)

    @property
    def url(self):
        return self.storage.url(self.name)

    @property
    def size(self):
        return self.storage.size(self.name) if self._committed else \
            self.file.size

    def open(self, mode=u'rb'):
        self.file.open(mode)
    # open() doesn't alter the file's contents, but it does reset the pointer
    open.alters_data = True

    @property
    def closed(self):
        file = getattr(self, u'_file', None)
        return file is None or file.closed

    def close(self):
        file = getattr(self, u'_file', None)
        if file is not None:
            file.close()


class MultiLingualFile(object):
    u"""
    A class that aggregates multiple files that each correspond to a separate
    language.

    Uses MultiLingualFieldFile instances (or None) for language-keyed
    attributes.
    """

    def __init__(self, xml=None, storage=None):
        u"""
        `storage` : a django storage class
        `xml` : An optional block of XML formatted like this:
        <languages>
            <language code="en">
                path/to/file.ext
            </language>
            <language code="es">
                path/to/file2.ext
            </language>
        </languages>

        If the above block of XML was passed (as `xml`) to an instance of
        MultiLingualText that instance would have two attributes:

        * `en` with a MultiLingualFieldFile instance that pulls
          `path/to/file.ext` from `storage`
        * `es` with a MultiLingualFieldFile instance that pulls
          `path/to/file2.ext` from `storage`
        """
        self.languages = LANGUAGES
        # Converting XML (passed-in as `xml`) to a python object via lxml
        if xml and storage:
            try:
                xml_as_python_object = objectify.fromstring(xml)
            except etree.XMLSyntaxError:
                raise Exception(INVALID_XML_ERROR + ' MultiLingualText')
            else:
                # Creating a dictionary of all the languages passed in the
                # value XML with the language code (i.e. 'en', 'de',
                # 'fr') as the key
                text_dict = {}
                try:
                    text_dict = dict(
                        (unicode(l.get(u'code')), unicode(l.text or u''))
                        for l in xml_as_python_object.language
                    )
                except AttributeError:
                    # Empty fields throw-off lxml and cause an AttributeError
                    pass
                for code, verbose in LANGUAGES:
                    setattr(
                        self,
                        code,
                        MultiLingualFieldFile(
                            storage=storage,
                            name=text_dict[code]
                        ) if code in text_dict else None
                    )
        else:
            for code, verbose in LANGUAGES:
                setattr(self, code, None)

    def __repr__(self):
        current = get_language()
        try:
            val = getattr(self, current)
        except AttributeError:
            if current not in [code for code, verbose in LANGUAGES]:
                raise ImproperlyConfigured(
                    UNKNOWN_LANGUAGE_CODE_ERROR.format(current)
                )
            return None
        return smart_str(val, errors='ignore')

    def __unicode__(self):
        return unicode(self.__repr__()) or u''

    def __nonzero__(self):
        """
        Provides 'truth value testing' based on the current language thread.

        Basically, if the language-specific attribute of a MultiLingualText
        instance associated with the current language thread would evaluate
        to `False` then calling `if` on the instance itself will evaluate to
        `False` as well.
        """
        if self.__unicode__():
            return True
        else:
            return False

    def as_xml(self):
        u"""Returns this instance as XML."""
        xml_to_return = etree.Element(u'languages')
        for key, value in self.__dict__.iteritems():
            if key != u'languages':
                language = etree.Element(u'language', code=key)
                language.text = value.name if value else u''
                xml_to_return.append(language)
        return etree.tostring(xml_to_return)



from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from django.core.files.base import File
from django.core.files.uploadedfile import (
    InMemoryUploadedFile, TemporaryUploadedFile
)
from django.forms import CharField, MultiValueField, ValidationError, FileField
from django.forms.widgets import FILE_INPUT_CONTRADICTION
from lxml import etree

from . import (
    widgets, LANGUAGES, LANGUAGES_REPLACEMENT,
    LANGUAGES_REQUIRED_TEXT, REQUIRED_ERROR
)

# This list is used to validate file uploads
FILE_FIELD_CLASSES = File.__subclasses__() + [
    TemporaryUploadedFile,
    InMemoryUploadedFile
]


class MultiLingualTextField(MultiValueField):
    u"""The field used by MultiLingualTextField."""
    widget = widgets.MultiLingualTextFieldWidget

    def widget_attrs(self, widget):
        u"""
        Given a Widget instance (*not* a Widget class), returns a dictionary
        of any HTML attributes that should be added to the Widget, based on
        this Field.
        """
        return {
            u'maxlength': self.individual_widget_max_length
        } if self.individual_widget_max_length else {}

    def __init__(self, *args, **kwargs):
        self.individual_widget_max_length = kwargs.get(
            u'individual_widget_max_length',
            None
        )
        if u'individual_widget_max_length' in kwargs:
            del kwargs[u'individual_widget_max_length']
        self.mandatory_field = kwargs[u'required']
        kwargs[u'required'] = False
        # Uncomment the next line when django 1.7 is released
        #kwargs[u'require_all_fields'] = kwargs[u'required'] = False
        fields = []
        for code, verbose in LANGUAGES:
            field = CharField(
                label=verbose,
                required=self.mandatory_field and code not in LANGUAGES_REPLACEMENT,
                max_length=self.individual_widget_max_length
            )
            field.error_messages.setdefault(
                u'incomplete',
                REQUIRED_ERROR.format(verbose)
            )
            fields.append(field)
        super(MultiLingualTextField, self).__init__(
            tuple(fields), *args, **kwargs
        )

    def compress(self, data_list):
        u"""
        Compresses a list of text into XML in the following structure:
        <languages>
            <language code="en">
                Hello
            </language>
            <language code="es">
                Hola
            </language>
        </languages>
        """
        xml = etree.Element(u'languages')
        if self.mandatory_field and not data_list:
            raise ValidationError(
                REQUIRED_ERROR.format(LANGUAGES_REQUIRED_TEXT)
            )
        elif data_list:
            for index, entry in enumerate(data_list):
                code, verbose = LANGUAGES[index]
                language = etree.Element(u'language', code=code)
                if code not in LANGUAGES_REPLACEMENT and not entry and self.mandatory_field:
                    raise ValidationError(REQUIRED_ERROR.format(verbose))
                language.text = entry
                xml.append(language)
        return etree.tostring(xml)


class MultiLingualCharField(MultiLingualTextField):
    u"""The field used by MultiLingualCharField."""
    widget = widgets.MultiLingualCharFieldWidget


class MultiLingualDjangoAdminTextField(MultiLingualTextField):
    u"""The field used by MultiLingualCharField."""
    widget = widgets.MultiLingualTextFieldDjangoAdminWidget


class MultiLingualDjangoAdminCharField(MultiLingualTextField):
    widget = widgets.MultiLingualCharFieldDjangoAdminWidget


class FileOrAlreadyExistantFilePathField(FileField):
    u"""
    A FileField subclass that provides either an instance of a 'File'
    (as determined by FILE_FIELD_CLASSES), a bool (which means the
    file is being 'cleared' from a model instance) or a str/unicode
    (a path to an existent file).
    """

    def clean(self, data, initial=None):
        # If the widget got contradictory inputs, we raise a validation error
        if not type(data) in File.__subclasses__():
            return data
        if data is FILE_INPUT_CONTRADICTION:
            raise ValidationError(self.error_messages[u'contradiction'])
        # False means the field value should be cleared; further validation is
        # not needed.
        if data is False:
            if not self.required:
                return False
            # If the field is required, clearing is not possible (the widget
            # shouldn't return False data in that case anyway). False is not
            # in validators.EMPTY_VALUES; if a False value makes it this far
            # it should be validated from here on out as None (so it will be
            # caught by the required check).
            data = None
        return initial if (not data and initial) else super(
            FileOrAlreadyExistantFilePathField, self
        ).clean(data)


class MultiLingualFileField(MultiValueField):
    u"""The field used by MultiLingualFileField."""
    widget = widgets.MultiLingualClearableFileInputWidget

    def widget_attrs(self, widget):
        u"""
        Given a Widget instance (*not* a Widget class), returns a dictionary
        of any HTML attributes that should be added to the Widget, based on
        this Field.
        """
        return {
            u'maxlength': self.individual_widget_max_length
        } if self.individual_widget_max_length else {}

    def __init__(self, *args, **kwargs):
        self.individual_widget_max_length = kwargs.get(
            u'individual_widget_max_length', None
        )
        if u'individual_widget_max_length' in kwargs:
            del kwargs[u'individual_widget_max_length']
        self.mandatory_field = kwargs[u'required']
        fields = [
            FileOrAlreadyExistantFilePathField(
                label=verbose,
                max_length=self.individual_widget_max_length
            )
            for code, verbose in LANGUAGES
        ]
        super(MultiLingualFileField, self).__init__(
            tuple(fields), *args, **kwargs
        )

    def compress(self, data_list):
        u"""
        Compresses a list of text into XML in the following structure:
        <languages>
            <language code="en">
                Hello
            </language>
            <language code="es">
                Hola
            </language>
        </languages>
        """
        #languages = [code for code, verbose in LANGUAGES]
        #xml = etree.Element(u'languages')
        if self.mandatory_field and not data_list:
            raise ValidationError(
                REQUIRED_ERROR.format(LANGUAGES_REQUIRED_TEXT)
            )
        elif data_list:
            for index, this_file in enumerate(data_list):
                code, verbose = LANGUAGES[index]
                if (code not in LANGUAGES_REPLACEMENT and self.mandatory_field
                    and not this_file and not type(this_file) in FILE_FIELD_CLASSES):
                    raise ValidationError(REQUIRED_ERROR.format(verbose))
        return data_list



from __future__ import absolute_import, division, print_function, unicode_literals

from classytags.arguments import Argument
from classytags.core import Options
from classytags.helpers import AsTag
from django import template
from django.conf import settings

register = template.Library()


class GetTranslationForContext(AsTag):
    u"""
    Retrieves the correct 'translation' for a MultiLingualText instance based
    on the language in the current thread (by calling
    `django.utils.translation.get_language()`).

    Can be used as a standalone tag::

        {% get_for_current_language object.title %}

    Or assigned to a context variable with as::

        {% get_for_current_language object.title as the_title %}
        {{ the_title }}
    """
    name = u'get_for_current_language'
    options = Options(
        Argument(u'attr', required=True),
        u'as',
        Argument(u'varname', required=False, resolve=False)
    )

    def get_value(self, context, attr):
        try:
            return getattr(
                attr,
                context.get(u'LANGUAGE_CODE', settings.LANGUAGES[0][0])
            )
        except AttributeError:
            return u''


class GetTranslationByLanguageCode(AsTag):
    u"""
    Retrieves a specific 'translation' for a MultiLingualText instance.

    Can be used as a standalone tag::

        {% get_trans_by_code object.title 'en' %}

    Or assigned to a context variable with as::

        {% get_trans_by_code object.title 'en' as the_title %}
        {{ the_title }}
    """
    name = u'get_trans_by_code'
    options = Options(
        Argument(u'attr', required=True),
        Argument(u'language_code', required=True),
        u'as',
        Argument(u'varname', required=False, resolve=False)
    )

    def get_value(self, context, attr, language_code):
        try:
            return getattr(attr, language_code)
        except AttributeError:
            return u''


register.tag(GetTranslationForContext)
register.tag(GetTranslationByLanguageCode)



from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from django.forms.widgets import (
    CheckboxInput, ClearableFileInput, HiddenInput,
    MultiWidget, Textarea, TextInput, FILE_INPUT_CONTRADICTION
)
from django.utils.encoding import force_text
from django.utils.html import conditional_escape, format_html
from django.utils.safestring import mark_safe
from lxml import objectify
from lxml.etree import XMLSyntaxError

from . import datastructures, LANGUAGES, INVALID_XML_ERROR


class WidgetWithLanguageAddOn(object):
    u"""
    A form widget which appends an <add-on> tag corresponding to a
    language in ``settings.LANGUAGES``.

    Add a class depending of the ``language_code``, maybe useful
    for some jQuery functions like ``toggle()``.
    """

    def __init__(self, attrs, language=None):
        self.language_code, self.label = language
        super(WidgetWithLanguageAddOn, self).__init__(attrs)

    def render(self, *args, **kwargs):
        # FIXME do something with self.label ?
        html = super(WidgetWithLanguageAddOn, self).render(*args, **kwargs)
        return mark_safe(
            u'<div class="input-prepend tab_element tab_link_{0}">'
            u'<span class="add-on control-label">{2}</span>{1}'
            u'</div>'.format(self.language_code, html, self.label)
        )


class WidgetWithLanguageLabel(object):
    u"""
    A form widget which prepends a <label> tag corresponding to a
    language in settings.LANGUAGES.

    Add a class depending of the ``language_code``, maybe useful for
    some jQuery functions like ``toggle()``.
    """

    def __init__(self, attrs, language=None):
        self.language_code, self.label = language
        super(WidgetWithLanguageLabel, self).__init__(attrs)

    def render(self, name, value, attrs=None):
        html = super(WidgetWithLanguageLabel, self).render(name, value, attrs)
        return mark_safe(
            (
                u'<div class="control-group tab_element tab_link{0}">'
                u'<label class="control-label">{1}</label>'
                u'<div class="controls">{2}</div></div>'
            ).format(self.language_code, self.label, html)
        )


class CustomClearableFileInput(ClearableFileInput):
    u"""
    Identical to `ClearableFileInput` only it includes an extra hidden field
    (`initial_file`) which passes along the existing file name of the file
    associated with this field (if one exists).
    """
    template_with_initial = (
        u'%(initial_file)s %(initial_text)s: '
        u'%(initial)s %(clear_template)s<br />%(input_text)s: %(input)s'
    )
    template_with_clear = (
        '<span class="clearable-file-input">'
        '%(clear)s'
        '<label for="%(clear_checkbox_id)s">%(clear_checkbox_label)s</label>'
        '</span>'
    )

    def initial_filename_name(self, name):
        u"""
        Given the name of the hidden initial_text input,
        return the HTML id for it.
        """
        return name + u'-initial'

    def render(self, name, value, attrs=None):
        substitutions = {
            u'initial_text': self.initial_text,
            u'input_text': self.input_text,
            u'clear_template': u'',
            u'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = u'%(input)s'
        substitutions[u'input'] = super(ClearableFileInput, self).render(
            name,
            value,
            attrs
        )

        if value and hasattr(value, u'url'):
            template = self.template_with_initial
            substitutions[u'initial'] = format_html(
                u'<a href="{0}">{1}</a>',
                value.url,
                force_text(value)
            )
            initial_filename_name = self.initial_filename_name(name)
            substitutions[u'initial_file'] = HiddenInput().render(
                initial_filename_name,
                value,
                attrs={u'id': initial_filename_name}
            )
            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)
                substitutions[u'clear_checkbox_name'] = conditional_escape(
                    checkbox_name
                )
                substitutions[u'clear_checkbox_id'] = conditional_escape(
                    checkbox_id
                )
                substitutions[u'clear'] = CheckboxInput().render(
                    checkbox_name,
                    False,
                    attrs={u'id': checkbox_id}
                )
                substitutions[u'clear_template'] = self.template_with_clear % \
                    substitutions

        return mark_safe(template % substitutions)

    def value_from_datadict(self, data, files, name):
        upload = super(CustomClearableFileInput, self).value_from_datadict(
            data,
            files,
            name
        )

        if not self.is_required and CheckboxInput().value_from_datadict(
                data,
                files,
                self.clear_checkbox_name(name)):
            # If the user contradicts themselves (uploads a new file AND checks
            # the "clear" checkbox), we return a unique marker object that
            # FileField will turn into a ValidationError. False signals to
            # clear any existing value, as opposed to just None
            return FILE_INPUT_CONTRADICTION if upload else False
        # Return the uploaded file or figure out the name of the hidden field
        # which holds the existant filename (if one exists)  If that hidden
        # field is in the widget, pass the file path of that file on, defaults
        # to None
        return upload or data.get(self.initial_filename_name(name), None)


class MultiLingualFieldBaseMixInWidget(object):
    u"""
    The 'base' multilingual field widget. Returns a widget (as specified by
    the `for_each_field_widget` attribute) for each language specified in
    settings.LANGUAGES.
    """

    for_each_field_widget = None

    def __init__(self, attrs=None):
        widgets = [
            self.for_each_field_widget(attrs, language=language)
            for language in LANGUAGES
        ]
        super(MultiLingualFieldBaseMixInWidget, self).__init__(widgets, attrs)

    def render(self, name, value, attrs=None):
        rendered_widget = super(
            MultiLingualFieldBaseMixInWidget, self
        ).render(name, value, attrs)
        return mark_safe(
            '<div class="multilingual-mod %s">%s</div>' % (
                self.__class__.__name__.lower(),
                rendered_widget
            )
        )


class MultiLingualTextFieldWidget(MultiLingualFieldBaseMixInWidget,
                                  MultiWidget):
    u"""
    A widget that returns a `Textarea` widget for each language specified
    in settings.LANGUAGES.
    """

    class FieldWidget(WidgetWithLanguageAddOn, Textarea):
        pass
    for_each_field_widget = FieldWidget

    def decompress(self, value):
        u"""
        Receives an instance of `MultiLingualText` (or a properly-formatted
        block of XML) and returns a list of values corresponding in position
        to the current ordering of settings.LANGUAGES.
        """
        text_dict = {}
        if value:
            # Both MultiLingualCharField and MultiLingualTextField instances
            # provide `MultiLingualText` instances by default but handling
            # for raw XML has been included for convenience.
            if isinstance(value, datastructures.MultiLingualText):
                text_dict = dict(
                    (code, getattr(value, code))
                    for code, verbose in LANGUAGES
                )
            else:
                # Converting XML (passed-in as `value`) to a python object
                # via lxml
                try:
                    xml_as_python_object = objectify.fromstring(value)
                except XMLSyntaxError:
                    raise Exception(
                        '%s MultiLingualTextFieldWidget.decompress()!' %
                        INVALID_XML_ERROR
                    )
                else:
                    # Creating a dictionary of all the languages passed in the
                    # value XML with the language code (i.e. 'en', 'de', 'fr')
                    # as the key
                    text_dict = dict(
                        (unicode(l.code), unicode(l.language_text or u''))
                        for l in xml_as_python_object.language
                    )
        # Returning text from XML tree in order dictated by LANGUAGES
        return [text_dict.get(code, u'') for code, verbose in LANGUAGES]


class MultiLingualCharFieldWidget(MultiLingualTextFieldWidget):
    u"""
    A widget that returns a `TextInput` widget for each language
    specified in settings.LANGUAGES.
    """

    class FieldWidget(WidgetWithLanguageAddOn, TextInput):
        pass
    for_each_field_widget = FieldWidget


class MultiLingualClearableFileInputWidget(MultiLingualFieldBaseMixInWidget,
                                           MultiWidget):
    u"""
    A widget that returns a `ClearableFileInput` widget for each
    language specified in settings.LANGUAGES.
    """

    class FieldWidget(WidgetWithLanguageAddOn, CustomClearableFileInput):
        pass
    for_each_field_widget = FieldWidget

    def decompress(self, value):
        u"""
        Receives an instance of `MultiLingualFile` and returns a list of
        broken-out-files corresponding in position to the current ordering
        of settings.LANGUAGES.
        """
        text_dict = dict(
            (code, getattr(value, code))
            for code, verbose in LANGUAGES
        ) if value else {}
        # Returning text from XML tree in order dictated by LANGUAGES
        return [text_dict.get(code) for code, verbose in LANGUAGES]


class MultiLingualFieldDjangoAdminBaseMixInWidget(object):
    u"""
    A mix-in class that provides django admin-specific styles
    to a MultiLingualField widget.
    """
    class Media:
        css = {
            'all': (
                'multilingualfield/css/multilingualfield-djangoadmin.css',
            ),
        }


class MultiLingualCharFieldDjangoAdminWidget(
        MultiLingualFieldDjangoAdminBaseMixInWidget,
        MultiLingualCharFieldWidget):
    pass


class MultiLingualTextFieldDjangoAdminWidget(
        MultiLingualFieldDjangoAdminBaseMixInWidget,
        MultiLingualTextFieldWidget):
    pass


class MultiLingualClearableFileInputDjangoAdminWidget(
        MultiLingualFieldDjangoAdminBaseMixInWidget,
        MultiLingualClearableFileInputWidget):
    pass



from __future__ import absolute_import, division, print_function, unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _

# Error messages
INVALID_ARGUMENT_ERROR = _(u"'{0}' is not a valid argument for {1}.")
INVALID_XML_ERROR = _(u'Invalid XML was passed to')
LANGUAGES_REQUIRED_ERROR = _(u'The `multilingualfield` app requires that `LANGUAGES` '
    u'(https://docs.djangoproject.com/en/dev/ref/settings/#languages) be set in your settings file.')
UNKNOWN_LANGUAGE_CODE_ERROR = _(u"django.utils.translation.get_language returned a language code "
    u"('{0}') not included in the `LANGUAGES` setting for this project. Either add an entry for the "
    u"'{0}' language code to `LANGUAGES` or change your `LANGUAGE_CODE` setting to match a language "
    u"code already listed in `LANGUAGES`.")
REQUIRED_ERROR = _(u'This multi-lingual field is required therefore you must provide content in {0}.')
XML_SYNTAX_ERROR = _(u"Multi Lingual field instances must be created with either an instance of "
    u"`multilingualfield.fields.MultiLingualText` or a block of XML in the following format:")

LANGUAGES = getattr(settings, u'LANGUAGES', None)
if not LANGUAGES:
    raise ImproperlyConfigured(LANGUAGES_REQUIRED_ERROR)

LANGUAGES_REPLACEMENT = getattr(settings, u'LANGUAGES_REPLACEMENT', {})
LANGUAGES_REQUIRED_TEXT = u'({0})'.format(u', '.join((v for c, v in LANGUAGES if c not in LANGUAGES_REPLACEMENT)))



from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import os
from django.conf import settings
from django.core.exceptions import FieldError
from django.core.files.storage import default_storage
from django.db.models import SubfieldBase, Field
from lxml import objectify, etree

from . import (
    datastructures, forms, LANGUAGES,
    INVALID_ARGUMENT_ERROR, XML_SYNTAX_ERROR
)


class MultiLingualTextField(Field):
    u"""
    A django TextField for storing multiple manually-written translations
    of the same piece of text.
    """
    description = (
        'Stores multiple manually-written translations of the '
        'same piece of text.'
    )

    __metaclass__ = SubfieldBase

    def __init__(self, *args, **kwargs):
        self.individual_widget_max_length = kwargs.get('max_length', None)
        self._db_type = kwargs.get('db_type', 'text')
        if self._db_type not in ['text', 'mediumtext', 'longtext']:
            raise FieldError(
                "Invalid db_type! Allowed choices are 'text', "
                "'mediumtext' (MySQL only) or 'longtext'."
            )
        else:
            try:
                del kwargs['db_type']
            except KeyError:
                pass

        if self.individual_widget_max_length:
            # Removing max_length so syncdb/south don't make a DB column
            # that's too small for future language additions
            del kwargs[u'max_length']
        super(MultiLingualTextField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return 'TextField'

    def db_type(self, connection):
        db_type = self._db_type
        if db_type == 'mediumtext':
            if connection.settings_dict['ENGINE'] == 'django.db.backends.mysql':
                pass
            else:
                db_type = 'longtext'
        return db_type

    def to_python(self, value):
        u"""
        Takes XML data from the database and converts it into an instance
        of MultiLingualText.
        """
        # Obviously MultiLingualText instances aren't stored in the database
        # but this conditional is there 'just-in-case' since a seralizer might
        # be utilized in the future for this field. -> Else : Use the XML
        # stored in the database to create a MultiLingualText instance
        if isinstance(value, datastructures.MultiLingualText):
            return value
        return datastructures.MultiLingualText(xml=value)

    def get_prep_value(self, value):
        u"""
        Converts an instance of MultiLingualText into what will ultimately be
        stored in the database, a block of XML in the following format:
        <languages>
            <language code="en">
                Hello
            </language>
            <language code="es">
                Hola
            </language>
        </languages>
        """
        # Checks to see if this is a `MultiLingualText` instance
        if isinstance(value, datastructures.MultiLingualText):
            # If it is, convert the instance to XML
            xml = value.as_xml()
        else:
            # Otherwise check to see if it is a valid block of XML
            try:
                objectify.fromstring(value)
            except etree.XMLSyntaxError:
                # If not, raise an Exception
                raise Exception(XML_SYNTAX_ERROR + """<languages>
    <language code="en">
        Hello
    </language>
    <language code="es">
        Hola
    </language>
</languages>
""")
            else:
                # Otherwise set `xml` to `value`
                xml = value
        return xml

    def formfield(self, **kwargs):
        # This is a fairly standard way to set up some defaults while letting
        # the caller override them.
        defaults = {
            u'form_class': forms.MultiLingualTextField,
            u'individual_widget_max_length': self.individual_widget_max_length
        }
        defaults.update(kwargs)
        return super(MultiLingualTextField, self).formfield(**defaults)


class MultiLingualCharField(MultiLingualTextField):
    u"""
    A django CharField for storing multiple manually-written translations
    of the same piece of text.
    """
    def formfield(self, **kwargs):
        # This is a fairly standard way to set up some defaults while letting
        # the caller override them.
        defaults = {
            u'form_class': forms.MultiLingualCharField,
            u'individual_widget_max_length': self.individual_widget_max_length
        }
        defaults.update(kwargs)
        return super(MultiLingualCharField, self).formfield(**defaults)


class MultiLingualFileField(Field):
    u"""
    A django FileField for storing multiple files (by language)
    in a single field.
    """
    description = (
        u'Stores multiple files, organized by language. An example use case: '
        u'I want to store a separate PDF for this model instance for each'
        u'language on the site.'
    )

    __metaclass__ = SubfieldBase

    def __init__(self, verbose_name=None, name=None,
                 upload_to=u'', storage=None, **kwargs):
        self.individual_widget_max_length = kwargs.get(u'max_length', None)
        if self.individual_widget_max_length:
            # Removing max_length so syncdb/south don't make a DB column that's
            # too small for future language additions
            del kwargs[u'max_length']

        for arg in (u'primary_key', u'unique'):
            if arg in kwargs:
                raise TypeError(
                    INVALID_ARGUMENT_ERROR.format(arg, self.__class__)
                )

        self.storage = storage or default_storage
        self.upload_to = upload_to
        if callable(upload_to):
            self.generate_filename = upload_to
        super(MultiLingualFileField, self).__init__(verbose_name, name, **kwargs)

    def db_type(self, connection):
        return u'text'

    def to_python(self, value):
        u"""
        Takes XML data from the database and converts it into
        an instance of MultiLingualFile.
        """
        if isinstance(value, datastructures.MultiLingualFile):
            return value
        elif isinstance(value, list):
            languages = [code for code, verbose in LANGUAGES]
            xml_block = etree.Element(u'languages')
            for index, this_file in enumerate(value):
                language = etree.Element(u'language', code=languages[index])
                # If `this_file` exists and is a 'File'
                if this_file and (type(this_file) in forms.FILE_FIELD_CLASSES):
                    # Figure out 'intended' file name and path
                    file_name_and_path = os.path.join(
                        self.upload_to,
                        this_file.name
                    )
                    # Create file using this field's storage (as provided by
                    # the parent fields 'formfield' method)
                    created_file_name = self.storage.save(
                        file_name_and_path,
                        this_file
                    )
                    # Finally, assign the created file name to the XML block
                    language.text = created_file_name
                # Otherwise...
                else:
                    # ...if it's a bool it means the field is being cleared,
                    # otherwise it's a filename and path.
                    language.text = u'' if isinstance(this_file, bool) else this_file
                xml_block.append(language)
            xml = etree.tostring(xml_block)
        else:
            xml = value
        return datastructures.MultiLingualFile(xml=xml, storage=self.storage)

    def get_prep_value(self, value):
        u"""
        Converts an instance of MultiLingualFile into what will ultimately be
        stored in the database, a block of XML in the following format:
        <languages>
            <language code="en">
                Hello
            </language>
            <language code="es">
                Hola
            </language>
        </languages>
        """
        # Checks to see if this is a `MultiLingualFile` instance
        return value.as_xml() if isinstance(
            value, datastructures.MultiLingualFile) else value

    def formfield(self, **kwargs):
        # This is a fairly standard way to set up some defaults while letting
        # the caller override them.
        defaults = {
            u'form_class': forms.MultiLingualFileField,
            u'individual_widget_max_length': self.individual_widget_max_length
        }
        defaults.update(kwargs)
        return super(MultiLingualFileField, self).formfield(**defaults)

if u'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules(
        [],
        [
            "^multilingualfield\.fields\.MultiLingualCharField",
            "^multilingualfield\.fields\.MultiLingualFileField",
        ]
    )
    multilingualtextfield_rules = [
        (
            (MultiLingualTextField,),
            [],
            {
                'db_type': ['_db_type', {'default': 'text'}]
            }
        )
    ]
    add_introspection_rules(
        multilingualtextfield_rules,
        ["^multilingualfield\.fields\.MultiLingualTextField", ]
    )



from django.contrib.admin import ModelAdmin

from . import widgets, fields


class MultiLingualFieldModelAdmin(ModelAdmin):
    formfield_overrides = {
        fields.MultiLingualCharField: {
            'widget': widgets.MultiLingualCharFieldDjangoAdminWidget
        },
        fields.MultiLingualTextField: {
            'widget': widgets.MultiLingualTextFieldDjangoAdminWidget
        },
        fields.MultiLingualFileField: {
            'widget': widgets.MultiLingualClearableFileInputDjangoAdminWidget
        },
    }



from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from . import fields, LANGUAGES


ARGUMENT = u'{0}__regex'
LANGUAGE_REGEX = u'.*<language code="{0}">[^<]+</language>.*'


class MultilingualFieldsMixin(object):
    u"""Add some utility methods related to the multilingual fields."""

    def is_translation_complete(self, language_code):
        u"""
        Return True if all multilingual fields of the object are not empty
        for the language ``language_code``.
        """
        for field in self.__class__.multilingual_fields():
            value = getattr(getattr(self, field.name), language_code)
            if value is None or len(value) == 0:
                return False
        return True

    @property
    def complete_translations(self):
        u"""
        Check the multilingual fields of the object and return a set with the
        languages that 'are' in all fields.

        .. todo:: a better explanation (a little bit tired today)
        """
        translations = set()
        for code, verbose in LANGUAGES:
            translations.add(code)
            for field in self.__class__.multilingual_fields():
                value = getattr(getattr(self, field.name), code)
                if value is None or len(value) == 0:
                    translations.discard(code)
                    break
        return translations

    @classmethod
    def multilingual_fields(cls):
        u"""
        Return a generator which returns the fields of the model that are
        instance of the multilingual fields.
        """
        mf1, mf2 = fields.MultiLingualTextField, fields.MultiLingualFileField
        return (f for f in cls._meta.fields if isinstance(f, mf1) or isinstance(f, mf2))

    @classmethod
    def objects_with_incomplete_translations(cls, language_code,
                                             fields_names=None, inverse=None):
        u"""
        Return a queryset filtering the objects that does not have all
        (or ``fields_names``) of his multilingual fields translated in
        ``language_code``.
        """
        if fields_names is None:
            fields_names = [f.name for f in cls.multilingual_fields()]
        language_regex = LANGUAGE_REGEX.format(language_code)
        arguments = dict(
            (ARGUMENT.format(name), language_regex)
            for name in fields_names
        )
        return cls.objects.filter(**arguments) if inverse else cls.objects.exclude(**arguments)



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(
    name='django-multilingualfield',
    packages=find_packages(),
    version='0.3.1',
    author=u'Jonathan Ellenberger',
    author_email='jonathan_ellenberger@wgbh.org',
    url='http://github.com/WGBH/django-multilingualfield/',
    license='MIT License, see LICENSE',
    description='A south-compatible suite of django fields that make it easy '
                'to manage multiple translations of text-based content '
                '(including files/images).',
    long_description=open('README.md').read(),
    zip_safe=False,
    install_requires=[
        'django-classy-tags>=0.3.4.1',
        'lxml>=3.1.2'
    ],
    package_data={
        'multilingualfield': [
            'static/multilingualfield/css/*.css'
        ]
    },
    classifiers=[
        'Framework :: Django',
    ]
)
